package Hawat::Controller::Log;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Data::Dumper;
use Mentat::Core qw(:c_hawat);

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Hawat::Controller::Secure'; }

__PACKAGE__->config(
    class       => 'Hawat::Controller::Log',
    constructor => 'new',
);

=head1 NAME

Hawat::Controller::Log - Catalyst controller for log management

=head1 DESCRIPTION

Catalyst controller for handling actions related to log management.

=head1 ACTIONS

=cut

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

=head2 index

Default controller action, redirects to the 'list' action

=cut

sub index :Path :Args(0) {
    my ( $self, $c ) = @_;
    $c->response->redirect($c->uri_for($self->action_for('list')));
}

=head2 list

View the list of all reports stored in the database

=cut

sub list :Local :Args(0) {
    my ($self, $c) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/find' action
    my $conf = $self->_dbicnf(
        limit  => $c->cfg('result_limit'),
        sortby => 'ts',
        sort   => 'desc',
    );
    $c->forward(HAWAT_DBI_FIND, [$conf]);

    my $fr = $c->stash->{logs};
    $c->stash(frs => $c->model('Logs')->calculate_frs($fr));

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Field', $variable) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 view

View the details of arbitrary log

=cut

#sub view :Local :Args(1) { # Switch to let the framework handle the error reporting
sub view :Local {
    my ($self, $c, $id) = @_;

    # Store the back link to this request
    $c->session->{backlink} = $c->req->uri;

    # Configure and forward to '/dbitem/fetch' action
    my $conf = $self->_dbicnf();
    $c->forward(HAWAT_DBI_FETCH, [$conf, $id]);

    # TODO: make use of 'nowrap' for ajax queries
    #$c->stash(nowrap => 1);

    # Optionally dump interesting variables - dependent on the DEVEL switch
    $self->dump_variable($c, 'Conf', $conf) if $c->cfg('devel');

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 delete

Delete one particular log

=cut

#sub delete :Local :Args(1) {# Switch to let the framework handle the error reporting
sub delete :Local {
    my ($self, $c, $id, $confirm) = @_;

    # Determine the backlink to which to return to
    my $backlink = $c->session->{backlink} || $c->uri_for('/log');

    # Configure and forward to '/dbitem/delete' action
    my $conf_d = $self->_dbicnf(
        backlink   => $backlink,
        backlink_d => $c->uri_for('/log/list'),
    );
    $c->forward(HAWAT_DBI_DELETE, [$conf_d, $id, $confirm]);

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

# _dbicnf(%)
#
#  Usage     :
#  Purpose   :
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub _dbicnf(%)
{
    my ($self, %cnf_a) = @_;
    my $cnf = {
            controller => 'Log',
            model      => HAWAT_MODEL_LOGS(),
            items      => 'Logs',
            item       => 'Log',
            menu       => 'admin',
        };
    foreach my $k (keys %cnf_a) {
        $cnf->{$k} = $cnf_a{$k};
    }
    return $cnf;
}

=head1 AUTHOR

 Jan Mach
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
