package Hawat::Controller::Root;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Mentat::Core qw(:c_hawat);

BEGIN { extends 'Hawat::Controller::Base' }

#
# Sets the actions in this controller to be registered with no prefix
# so they function identically to actions created in MyApp.pm
#
__PACKAGE__->config(namespace => '');

=head1 NAME

Hawat::Controller::Root - Root controller for Mentat web GUI

=head1 DESCRIPTION

[enter your description here]

=head1 ACTIONS

=head2 index

Default controller action, currently redirects user to the 'welcome' action

=cut

sub index :Path :Args(0) {
    my ($self, $c) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    # Perform redirection to welcome page
    $c->response->redirect($c->uri_for($c->controller('root')->action_for('welcome')));
}

=head2 default

Standard error 404 page

=cut

sub default :Path {
    my ($self, $c) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    # Display 404 - Page not found instead of current content
    $c->detach(HAWAT_ERROR_HANDLER, [message => '404 - Page not found',
                                     status  => 404]);
}

=head2 welcome

Welcome page

=cut

sub welcome :Global {
    my ($self, $c) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    # Choose the appropriate template and let it be rendered
    $c->stash(template => 'root/welcome');
}

=head2 login

Login page and application logic

=cut

sub login : Global {
    my ( $self, $c ) = @_;

    # Gather user information provided by IdP
    $c->forward('gather_uinfo_env');

    # Attempt to authenticate the user
    if ($c->authenticate()) {

        $self->flash_success($c, "You have been successfully logged in as '".$c->user->{_id}."'");

        # User account must be activated before it can be used
        unless ($c->user_is_activated()) {
            # Dump a log message to the development server debug output
            $c->log->debug('***Root::login User not activated, forwarding to /activation');
            $c->detach('/activation');
        }

        # Update account data after login
        $c->forward('update_account_data');

        # If the login was forced, redirect back to the previous page
        if ($c->flash->{redirect_after_login}) {
            $c->response->redirect($c->flash->{redirect_after_login});
            return;
        }

        # Perform redirection to some meaningfull page, e.g. user darshboard etc.
        $c->response->redirect($c->uri_for($c->controller('Alert')->action_for('list')));
        return;

    } else {

        # Set an error message
        $self->flash_warning($c, "User account with your credentials does not exist. Please create a new one and wait for activation.");
        $c->response->redirect($c->uri_for('register'));
        return;
    }

    # Choose the template
    $c->stash(template => 'root/login');
}

=head2 logout

Logout page and application logic

=cut

sub logout : Global {
    my ( $self, $c ) = @_;

    # Clear the user's state
    $c->logout;

    # Set a flash message
    $self->flash_success($c, "You have been successfully logged out.");

    # Or choose the template and display some logout content
    $c->stash(template => 'root/logout');

    # Send the user to the starting point
    $c->response->redirect($c->uri_for('/'));
    return;
}

=head2 register

Registration page and application logic

=cut

sub register : Global {
    my ( $self, $c ) = @_;

    # Gather user information provided by IdP
    $c->forward('gather_uinfo_env');

    # Gather user information provided by IdP
    $c->forward('check_account_existence');

    # Fetch the form data (submitted using POST method)
    my $form = $c->request->body_params;

    # If the form was correctly submitted, attempt to perform the registration
    if ($form->{submitted}) {
        # Check for cancelation button
        if ($form->{submit} eq HAWAT_FORM_CANCEL()) {
            $self->flash_info($c, "Account registration canceled.");
            $c->response->redirect($c->uri_for('/'));
            return;
        }

        $self->dump_variable($c, 'User info: ', $c->stash->{uinfo_env});
        $self->dump_variable($c, 'Registration form: ', $form);
        my ($profile, $errors, $rv) = $c->model('Users')->register($c->stash->{uinfo_env}, $form, $c->list_roles());
        $c->stash(uinfo => $profile);

        if ($errors and @$errors) {
            $c->stash('register_form_errors' => $errors);
        }
        else {
            if ($rv) {
                $self->flash_success($c, "Your user account was successfully registered.");
                $c->forward('registration_inform', [$profile]);
                $c->detach('/activation');
            }
            else {
                $self->flash_error($c, "Error registering your user account, please try again.");
            }
        }
    }
    else {
        $c->stash(uinfo => $c->stash->{uinfo_env});
    }

    # Choose the template
    $c->stash(template => 'root/register');
}

=head2 activation

Page displaying information about necessary account activation

=cut

sub activation : Local {
    my ( $self, $c ) = @_;
    my $message = "Your user account was created and is awaiting activation. System administrator was informed and will take care of this matter as soon as possible. Please wait.";
    $c->detach(HAWAT_MSG_HANDLER, [message => $message,
                                   type    => 'warning',
                                   menu    => 'welcome',
                                   title   => 'Pending account activation']);
}

=head2 badidp

Page displaying information about bad identity provider

=cut

sub badidp : Local {
    my ( $self, $c ) = @_;
    my $message = "Identity provider you have chosen does not provide the requested 'eppn' attribute, authentication with this site will not be possible. Please contact your IdP to fix this, or choose another one, if possible.";
    $c->detach(HAWAT_MSG_HANDLER, [message => $message,
                                   type    => 'error',
                                   menu    => 'welcome',
                                   title   => 'Bad IdP']);
}

=head2 unauthorized

Page displaying information about user not being logged in when mandatory

=cut

sub unauthorized : Local {
    my ( $self, $c ) = @_;
    $c->detach(HAWAT_ERROR_HANDLER, [message => 'Permission denied, you are not logged in. Please log in and try again.',
                                     status  => 401]);
}

=head2 forbidden

Page displaying information about insufficient access rights

=cut

sub forbidden : Local {
    my ( $self, $c ) = @_;
    $c->detach(HAWAT_ERROR_HANDLER, [message => 'Permission denied, you are not allowed to perform requested action.',
                                     status  => 403]);
}

#-------------------------------------------------------------------------------
# Private actions
#-------------------------------------------------------------------------------

=head2 gather_uinfo_env

Gather user information from server environment

=cut

sub gather_uinfo_env : Private {
    my ($self, $c) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    # Fetch the user information from environment
    my $lenv = $c->engine->env;
    # Prepare the storage for gathered user data
    my $uinfo = {};

    #$c->forward('badidp') unless $lenv->{eppn};

    # Fill in the user data
    $uinfo->{id}   = $lenv->{eppn};
    $uinfo->{name} = $lenv->{cn};
    if ($lenv->{perunPreferredMail}) { $uinfo->{email} = $lenv->{perunPreferredMail}; }
    elsif ($lenv->{mail})            { $uinfo->{email} = (split(';', $lenv->{mail}))[0]; }
    if ($lenv->{o})                        { $uinfo->{organization} = $lenv->{o}; }
    elsif ($lenv->{perunOrganizationName}) { $uinfo->{organization} = $lenv->{perunOrganizationName}; }
    if ($lenv->{affiliation}) { $uinfo->{affiliations} = [split(';', $lenv->{affiliation})]; }
    if ($lenv->{perunUniqueGroupName}) { $uinfo->{orggroups} = [split(';', $lenv->{perunUniqueGroupName})]; }

    # Stash it for later use
    $c->stash(uinfo_env => $uinfo);
}

=head2 check_account_existence

Check the existence of user account

=cut

sub check_account_existence : Private {
    my ($self, $c) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    my $username = $c->stash->{uinfo_env}->{id};
    my ($user, $count, $duration) = $c->model('Users')->fetch_account($username);
    if ($user) {
        my $message = "Your account '$username' already exists.";
        $c->detach(HAWAT_MSG_HANDLER, [message => $message,
                                       type    => 'warning',
                                       menu    => 'welcome',
                                       title   => 'Account already exists']);
    }
}

=head2 update_account_data

Update account data after each login

=cut

sub update_account_data : Private {
    my ($self, $c) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    my $username = $c->user->{_id};
    eval {
        my ($user, $count, $duration) = $c->model('Users')->fetch_account($username);
        if ($user) {
            my $ui = $c->stash->{uinfo_env};
            if ($user->{ts_last_login}) {
                $c->stash->{last_login} = $user->{ts_last_login};
            }
            else {
                $user->{HAWAT_ATTR_EMAIL()} = $ui->{email}        if $ui->{email};
                $user->{HAWAT_ATTR_ORG()}   = $ui->{organization} if $ui->{organization};
            }
            $user->{HAWAT_ATTR_ORGGROUPS()} = $ui->{orggroups}    || [];
            $user->{HAWAT_ATTR_AFFIL()}     = $ui->{affiliations} || [];
            $user->{ts_last_login}          = time;
            my $rv = $c->model('Users')->update($username, $user, $c->list_roles());
        }
    };
    if ($@) {
        $c->log->error("Unable to update user record: '$@'");
    }
}

=head2 registration_inform

Inform administrators about account registration

=cut

sub registration_inform : Private {
    my ($self, $c, $profile) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    my $username = $profile->{HAWAT_ATTR_MID()};

    my $message = "New account '$username' was just registered to Mentat system. Please review the following information and activate or delete the account:\n\n";
    $message .= "    username:            ".$profile->{HAWAT_ATTR_MID()}."\n";
    $message .= "    name:                ".$profile->{HAWAT_ATTR_NAME()}."\n";
    $message .= "    email:               ".$profile->{HAWAT_ATTR_EMAIL()}."\n";
    $message .= "    organization:        ".$profile->{HAWAT_ATTR_ORG()}."\n";
    if ($profile->{HAWAT_ATTR_ORGGROUPS()} and ref $profile->{HAWAT_ATTR_ORGGROUPS()} eq 'ARRAY' and @{$profile->{HAWAT_ATTR_ORGGROUPS()}}) {
        $message .= "    organization groups: \n";
        foreach my $item (@{$profile->{HAWAT_ATTR_ORGGROUPS()}}) {
            $message .= "        - $item\n";
        }
    }
    if ($profile->{HAWAT_ATTR_AFFIL()} and ref $profile->{HAWAT_ATTR_AFFIL()} eq 'ARRAY' and @{$profile->{HAWAT_ATTR_AFFIL()}}) {
        $message .= "    known affiliations: \n";
        foreach my $item (@{$profile->{HAWAT_ATTR_AFFIL()}}) {
            $message .= "        - $item\n";
        }
    }
    $message .= "Account details can be found here:\n  ".$c->uri_for($c->controller('user')->action_for('view'), $username)."\n";
    $message .= "Direct activation link:\n  ".$c->uri_for($c->controller('user')->action_for('unlock'), $username)."\n";
    $message .= "Direct deletion link:\n  ".$c->uri_for($c->controller('user')->action_for('delete'), $username)."\n\n";
    $message .= "Have a nice day\n\n--\nMentat System\n";

    $c->log->debug("Created new account for user '$username'");
    $c->email(
            header => [
                From    => 'hawat@mentat-hub.cesnet.cz',
                To      => 'mentat-admin@mentat-hub.cesnet.cz',
                Subject => "Mentat: New account '$username' was just registered",
            ],
            body => $message,
        );
}

=head2 activation_inform

Inform users about account activation

=cut

sub activation_inform : Private {
    my ($self, $c, $id) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    my ($message, $to);
    $c->stash('account_id' => $id);
    $message = $c->forward($c->view('Text'), "render", ['activation.txt.tt']);

    $to = $c->stash->{user}->{email};
    $to = 'mentat-admin@mentat-hub.cesnet.cz' unless $to;

    $c->log->debug("Activated account for user '$id'");
    $c->email(
            header => [
                From    => 'hawat@mentat-hub.cesnet.cz',
                To      => $to,
                CC      => 'mentat-admin@mentat-hub.cesnet.cz',
                Subject => "[Mentat] Aktivace uctu '$id'",
            ],
            body => $message,
        );
}

=head2 hnd_message

Handle message - display abitrary message instead of page content

=cut

sub hnd_message : Private {
    my ($self, $c, %args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);
    $args{template} = 'inc/dialog_message_generic.html.tt' unless $args{template};
    $args{type}     = 'info' unless $args{type};

    $c->stash(template => $args{template},
              message  => $args{message},
              msg_type => $args{type},
        );

    $c->stash(menu      => $args{menu})  if $args{menu};   # Optionally choose the active main menu item
    $c->stash(pagetitle => $args{title}) if $args{title};  # Optionally set the page title from here

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 hnd_error

Handle the error - display it on the current page

=cut

sub hnd_error : Private {
    my ($self, $c, %args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);
    $args{template} = 'inc/dialog_error_generic.html.tt' unless $args{template};
    $args{message}  = 'Something went terribly wrong' unless $args{message};
    $args{type}     = 'danger'  unless $args{type};
    $args{menu}     = 'welcome' unless $args{menu};

    # Store the error to catalyst internal error storage
    #$c->error($error);

    # Choose the error template and store the error text
    $c->stash(template => $args{template},
              errors   => $args{message},
              msg_type => $args{type},
        );
    $c->stash(errors_dev => $args{message_dev}) if $args{message_dev}; # Optionally choose the more concrete message for developer
    $c->stash(menu       => $args{menu})        if $args{menu};        # Optionally choose the active main menu item
    $c->stash(pagetitle  => $args{title})       if $args{title};       # Optionally set the page title from here

    # Set the appropriate response status
    $c->stash(status_code => $args{status}) if $args{status}; # Optionally set the status code
    $c->response->status($args{status})     if $args{status}; # Optionally set the status code

    # Render the view
    # NOTE: No need for that now, it will be rendered automagically, when the
    # 'end' method is called (see the Hawat::Controller::Root for details)
    #$c->forward($c->view(HAWAT_VIEW_HTML));
}

=head2 hnd_error_redir

Handle the error - display it on the page user is redirected to

=cut

sub hnd_error_redir : Private {
    my ($self, $c, %args) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);
    $args{message}  = 'Something went terribly wrong' unless $args{message};
    $args{backlink} = '/' unless $args{backlink};

    # Use the catalyst loging facilities
    #$c->log->error($error);
    #$c->error($error);

    $self->flash_error($c, $args{message});
    $c->response->redirect($args{backlink});
}

=head2 end

Attempt to render a view, if needed, customized to be able to display other errors.

=cut

sub end : Private {
    my ( $self, $c ) = @_;
    confess "Instance method not invoked on object instance" unless blessed($self);
    confess "Context must be object instance" unless blessed($c);

    eval {
        my $u = ($c->user_exists) ? $c->user->id : 'anonymous';
        $c->model('Accesslogs')->create($c->req->path, $u, $c->req->uri);
    };
    if ($@) {
        $c->log->error("Unable to update access log: '$@'");
    }

    # Optionally dump interesting variables - dependent on the DEVEL switch
    #$self->dump_variable($c, 'Variable', $variable) if $c->cfg('devel');

    # Custom error reporting
    if (scalar @{$c->error}) {
        for my $error (@{$c->error}) {
            $c->log->error($error);
        }
        $c->stash(template => 'inc/dialog_error_generic.html.tt',  # Use the generic error template page
                  errors => $c->error,                             # Stash the error message
        );
        $c->forward($c->view(HAWAT_VIEW_HTML));
        $c->clear_errors;
    }

    # Some responses do not need any content
    return 1 if $c->response->status =~ /^3\d\d$/; # Redirection, not modified, etc.
    return 1 if $c->response->status =~ /^204$/;   # No content
    # Do not render anything, if it was already rendered
    return 1 if $c->response->body;

    # Make sure the content type header is always set
    unless ( $c->response->content_type ) {
        $c->response->content_type('text/html; charset=utf-8');
    }

    # Render the view
    $c->log->debug("Auto-rendering view: ".(($c->stash->{template})?$c->stash->{template}:'default'));
    $c->forward($c->view(HAWAT_VIEW_HTML));
}

=head1 AUTHOR

 Jan Mach, Radomír Orkáč
 Cesnet, z.s.p.o
 jan.mach@cesnet.cz, radomir.orkac@cesnet.cz
 http://www.cesnet.cz

=head1 LICENSE

This library is free software. You can redistribute it and/or modify
it under the same terms as Perl itself.

=cut

__PACKAGE__->meta->make_immutable;

1;
