package Hawat::Model::Whois;
use strict;
use warnings;
use Moose;
use namespace::autoclean;

use Carp;
use Scalar::Util 'blessed';
use Time::HiRes qw (time);

use Log::Loger;
use Mentat::Core qw(:c_hawat);
use DNS::Resolver;
use Value::Convertor;
use Mentat::Storage::Mongo;
use Mentat::Mailer::Report;
use Whois::Service;
use Whois::Mongo;
use Whois::Negistry;
use Whois::Remote;

#-------------------------------------------------------------------------------
# Initialization section
#-------------------------------------------------------------------------------

BEGIN { extends 'Catalyst::Model::Adaptor'; }

__PACKAGE__->config(
    class            => 'Hawat::Model::Whois',
    constructor      => 'new',
    devel            => 0,
    database_name    => 'mentat',
    collection_name  => 'groups',
    whois_dump_file  => '/var/mentat/whois-negistry.json',
    test_mode        => 0,
);
has "devel"            => (is => 'ro', isa => 'Bool', required => 1);
has "database_name"    => (is => 'ro', isa => 'Str',  required => 1);
has "collection_name"  => (is => 'ro', isa => 'Str',  required => 1);
has "test_mode"        => (is => 'ro', isa => 'Bool', required => 1);
has "whois_dump_file"  => (is => 'ro', isa => 'Str',  required => 1);
has "whois_exceptions" => (is => 'ro', isa => 'HashRef');
has "db"               => (is => 'rw', isa => 'Mentat::Storage::Mongo');

#-------------------------------------------------------------------------------
# Implementation section
#-------------------------------------------------------------------------------

# search($)
#
#  Usage     : my ($whois, $count, $duration) = $c->model('Whois')->search($query);
#  Purpose   : Get one particular report within the report storage
#  Arguments :
#  Returns   :
#  Throws    : Dies, if not invoked on object instance

sub search($)
{
    my $self = shift;
    confess "Instance method not invoked on object instance" unless blessed($self);
    my ($query,) = @_;
    die "Query must be given as argument\n" unless $query;

    my $start = time();

    # Prepare various whois service objects
    my $whois_local = Whois::Service->new(
            Whois::Mongo->new($self->database_name, $self->collection_name, 300),
        );
    my $whois_negistry = Whois::Negistry->new($self->whois_dump_file, 1, $self->whois_exceptions);
    my $whois_remote   = Whois::Remote->new();

    my (@results_local, @results_negistry, @results_remote, $type);
    # Query whois services - Reverse queries
    if ($query =~ /@/) {
        $type             = 'email';
        @results_local    = $whois_local->rfind($query);
        @results_negistry = $whois_negistry->rfind($query);
        @results_remote   = $whois_remote->rfind($query);
    }
    # Query whois services - Normal whois queries
    else {
        $type             = 'ip';
        @results_local    = $whois_local->find($query);
        @results_negistry = $whois_negistry->find($query);
        @results_remote   = $whois_remote->find($query);
    }

    # Prepare result
    my $r = {
            'local'    => \@results_local,
            'negistry' => \@results_negistry,
            'remote'   => \@results_remote
        };
    my $duration = Value::Convertor->duration_to_string(time() - $start);

    return ($type, $r, $duration);
}

#-------------------------------------------------------------------------------

# BUILD()
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Moose::Manual::Construction(3) for details

sub BUILD {
    my $self = shift;

    # Initialize the internal database connection for whole model
    $self->db(Mentat::Storage::Mongo->new(
                    database => $self->database_name,
                    collection => $self->collection_name));

    # If you use 'weak_ref' for moose attribute, previous call will not set the
    # attribute. It is necessary to do it with antermediate variable $self->db($r);
    die "Unable to initialize database connection attribute\n" unless $self->db;
}

# COMPONENT
#
#  Purpose : Perform additional instance setup after the object is created
#  Comment : Callback method, see Catalyst::Component(3) for details
#
#  FIXME: For some reason config merging does not work without this code present
#         in the package!!!

sub COMPONENT {
    my ($class, $app, $args) = @_;
    $args = $class->merge_config_hashes($class->config, $args);
    #use Data::Dumper;
    #$app->log->debug("X $class: " . Dumper($class->config));
    #$app->log->debug("Y args  : " . Dumper($args));
    #$app->log->debug("Z $app  : " . Dumper($app->config));
    #$app->log->debug("MERGE XY: " . Dumper($args));
    return $class->new($app, $args);
}

1;
